package main

import (
	"cidebug/pkg/converge"
	"cidebug/pkg/iowrappers"
	"fmt"
	"github.com/gorilla/websocket"
	"log"
	"net/http"
	"regexp"
)

var upgrader = websocket.Upgrader{
	ReadBufferSize:  1024,
	WriteBufferSize: 1024,
}

func handleWebSocket(w http.ResponseWriter, r *http.Request,
	handler func(w http.ResponseWriter, r *http.Request, websockerConnection iowrappers.ReadWriteAddrCloser)) {
	conn, err := upgrader.Upgrade(w, r, nil)
	if err != nil {
		log.Println("Error upgrading to WebSocket:", err)
		return
	}
	wsConn := iowrappers.NewWebSocketConn(conn)
	defer wsConn.Close()

	handler(w, r, wsConn)
}

type WebSocketService struct {
	handler func(w http.ResponseWriter, r *http.Request, conn iowrappers.ReadWriteAddrCloser)
}

func (endpoint *WebSocketService) handle(w http.ResponseWriter, r *http.Request) {
	handleWebSocket(w, r, endpoint.handler)
}

func parsePublicId(path string) (publicId string, _ error) {
	pattern := regexp.MustCompile("^/[^/]+/([^/]+)$")
	matches := pattern.FindStringSubmatch(path)
	if len(matches) != 2 {
		return "", fmt.Errorf("Invalid URL path '%s'", path)
	}
	return matches[1], nil
}

func main() {

	admin := converge.NewAdmin()
	registrationService := WebSocketService{
		handler: func(w http.ResponseWriter, r *http.Request, conn iowrappers.ReadWriteAddrCloser) {
			publicId, err := parsePublicId(r.URL.Path)
			if err != nil {
				log.Printf("Cannot parse public id from url: '%v'\n", err)
				return
			}
			log.Printf("Got registration connection: '%s'\n", publicId)
			err = admin.Register(publicId, conn)
			if err != nil {
				log.Printf("Error %v\n", err)
			}
		},
	}
	clientService := WebSocketService{
		handler: func(w http.ResponseWriter, r *http.Request, conn iowrappers.ReadWriteAddrCloser) {
			publicId, err := parsePublicId(r.URL.Path)
			if err != nil {
				log.Printf("Cannot parse public id from url: '%v'\n", err)
				return
			}
			log.Printf("Got client connection: '%s'\n", publicId)
			err = admin.Connect(publicId, conn)
			if err != nil {
				log.Printf("Error %v\n", err)
			}
		},
	}

	http.HandleFunc("/agent/", registrationService.handle)
	http.HandleFunc("/client/", clientService.handle)

	// Start HTTP server
	fmt.Println("Rendez-vous server listening on :8000")
	log.Fatal(http.ListenAndServe(":8000", nil))
}
